@@ -12,12 +12,12 @@
 ##}
 
 .onAttach <- function(...){
-  packageStartupMessage("Classes and Methods for R developed in the\n",
+  packageStartupMessage("Classes and Methods for R originally developed in the\n",
                         "Political Science Computational Laboratory\n",
                         "Department of Political Science\n",
-                        "Stanford University\n",
-                        "Simon Jackman\n",
-                        "hurdle and zeroinfl functions by Achim Zeileis")
+                        "Stanford University (2002-2015),\n",
+                        "by and under the direction of Simon Jackman.\n",
+                        "hurdle and zeroinfl functions by Achim Zeileis.")
 }
 
 .onUnload <- function(libpath){
